#ifndef AODTOLIVERPOOLNTUPLE_H
#define AODTOLIVERPOOLNTUPLE_H

// -------------------------------------------------------
// AODtoLiverpoolNtuple
//
// Purpose: Making an Ntuple of quantities from the AOD.
//
// Author: Barry King
// Last changed by:Ellie Dobson
// 06-02-07
// -------------------------------------------------------

#include "GaudiKernel/Algorithm.h"
#include "GaudiKernel/ObjectVector.h"
#include "CLHEP/Units/SystemOfUnits.h"
#include "StoreGate/StoreGateSvc.h"
#include "GaudiKernel/NTuple.h"
#include <string>

//class IHistogram1D;
//class IHistogram2D;
class IAnalysisTools;
class egammaqpoint;
class egammaqgcld;
class AODtoLiverpoolNtuple : public Algorithm {
 
 public:

   AODtoLiverpoolNtuple(const std::string& name, ISvcLocator* pSvcLocator);
   ~AODtoLiverpoolNtuple();

   StatusCode initialize();
   StatusCode finalize();
   StatusCode execute();

 protected:
  // --------------------------------------------------
  // Book quantities
  // --------------------------------------------------

  StatusCode BookElectron(NTuplePtr nt);     
  StatusCode BookMuidMuons(NTuplePtr nt);	    
  StatusCode BookStacoMuons(NTuplePtr nt);	    
  StatusCode BookPhoton(NTuplePtr nt);	    
  StatusCode BookCTP_Decision(NTuplePtr nt);	    
  StatusCode BookTriggerDecision(NTuplePtr nt);	    
  StatusCode BookEFegamma(NTuplePtr nt);	    
  StatusCode BookTrigElectron(NTuplePtr nt);	    
  StatusCode BookL1EMTauObject(NTuplePtr nt);	    
  StatusCode BookL1JetObject(NTuplePtr nt);	    
  StatusCode BookL1EtmissObject(NTuplePtr nt);	    
  StatusCode BookEtMiss(NTuplePtr nt);
  StatusCode BookEtMissRefFinal(NTuplePtr nt);//ED
  StatusCode BookEtMissLocHadTopoObj(NTuplePtr nt);//ED
  StatusCode BookEtMissObjFinal(NTuplePtr nt);//ED	    
  StatusCode BookTauJet(NTuplePtr nt);	    
  StatusCode BookTauJet1p3p(NTuplePtr nt);	    
  StatusCode BookTrack(NTuplePtr nt);	    
  StatusCode BookJet(NTuplePtr nt);	    
  StatusCode BookTruth(NTuplePtr nt);	    
  StatusCode BookTruthParticle(NTuplePtr nt);
  StatusCode BookTruthJet(NTuplePtr nt);	    
  StatusCode BookTruthEtMiss(NTuplePtr nt);  
  StatusCode BookPrimaryVertex(NTuplePtr nt);

  // --------------------------------------------------
  // Unpack quantities
  // --------------------------------------------------

  StatusCode UnpackElectron();     
  StatusCode UnpackMuidMuons();	 
  StatusCode UnpackStacoMuons();	 
  StatusCode UnpackPhoton();	 
  StatusCode UnpackCTP_Decision();	 
  StatusCode UnpackTriggerDecision();	 
  StatusCode UnpackEFegamma();	 
  StatusCode UnpackTrigElectron();	 
  StatusCode UnpackL1EMTauObject();	 
  StatusCode UnpackL1JetObject();	 
  StatusCode UnpackL1EtmissObject();	 
  StatusCode UnpackEtMiss();
  StatusCode UnpackEtMissRefFinal();//ED
  StatusCode UnpackEtMissLocHadTopoObj();//ED
  StatusCode UnpackEtMissObjFinal();//ED	 
  StatusCode UnpackTauJet();	 
  StatusCode UnpackTauJet1p3p();	 
  StatusCode UnpackTrack();	 
  StatusCode UnpackJet();		 
  StatusCode UnpackTruth();	 
  StatusCode UnpackTruthParticle();
  StatusCode UnpackTruthJet();	 
  StatusCode UnpackTruthEtMiss();	 
  StatusCode UnpackPrimaryVertex();

 private:

  IAnalysisTools * m_analysisTools;
  StoreGateSvc*    m_storeGate;
  bool             m_fastsim;
  int              m_release;
  bool             m_doElectronTrigger;
  NTuplePtr        nt;

  // --------------------------------------------------
  // Ntuple entries
  // --------------------------------------------------

  // Version
  NTuple::Item<float> m_h_version_h;
  NTuple::Item<float> m_h_version_c;
  
  // Event
  NTuple::Item<long> m_h_event_runnum;
  NTuple::Item<long> m_h_event_eventnum;

  // Electron
  static const int max_electron_num=300;
  NTuple::Item<long>    m_h_electron_num;
  // Electron kinematics
  NTuple::Array<float> m_h_electron_pt;
  NTuple::Array<float> m_h_electron_eta;
  NTuple::Array<float> m_h_electron_phi;
  NTuple::Array<float> m_h_electron_e; 
  // Electron track info 
  NTuple::Array<long>  m_h_electron_hasTrack;
  NTuple::Array<float> m_h_electron_track_pt;
  NTuple::Array<float> m_h_electron_track_eta;
  NTuple::Array<float> m_h_electron_track_phi;
  NTuple::Array<float> m_h_electron_track_e;
  NTuple::Array<float> m_h_electron_track_d0;
  NTuple::Array<float> m_h_electron_track_z0;   
  NTuple::Array<float> m_h_electron_track_charge;
  NTuple::Array<long>  m_h_electron_numberOfBLayerHits;   
  NTuple::Array<long>  m_h_electron_numberOfPixelHits;  
  NTuple::Array<long>  m_h_electron_numberOfSCTHits;    
  NTuple::Array<long>  m_h_electron_numberOfTRTHits;   
  NTuple::Array<long>  m_h_electron_numberOfTRTHighThresholdHits;         
  // Electron algorithm
  NTuple::Array<long>  m_h_electron_egamma;        
  NTuple::Array<long>  m_h_electron_softe;        
  // EgammaPID parameters
  NTuple::Array<float> m_h_electron_IsEM;
  NTuple::Array<float> m_h_electron_ElectronWeight;     
  NTuple::Array<float> m_h_electron_BgWeight;     
  NTuple::Array<float> m_h_electron_Hmatrix; 
  NTuple::Array<float> m_h_electron_NeuralNet;     
  NTuple::Array<float> m_h_electron_SofteIsEM;
  NTuple::Array<float> m_h_electron_SofteElectronWeight;     
  NTuple::Array<float> m_h_electron_SofteBgWeight;     
  NTuple::Array<float> m_h_electron_SofteNeuralNet;     
  // Electron Parameters
  NTuple::Array<float> m_h_electron_EoverP;
  NTuple::Array<float> m_h_electron_etcone;  
  NTuple::Array<float> m_h_electron_etcone20;
  NTuple::Array<float> m_h_electron_etcone30;
  NTuple::Array<float> m_h_electron_etcone40;
  // for egamma electrons 
  NTuple::Array<float> m_h_electron_etaBE2;
  NTuple::Array<float> m_h_electron_et37;
  NTuple::Array<float> m_h_electron_e233;
  NTuple::Array<float> m_h_electron_e237;
  NTuple::Array<float> m_h_electron_e277;
  NTuple::Array<float> m_h_electron_ethad1;
  NTuple::Array<float> m_h_electron_weta1;
  NTuple::Array<float> m_h_electron_weta2;
  NTuple::Array<float> m_h_electron_f1;
  NTuple::Array<float> m_h_electron_e2tsts1;
  NTuple::Array<float> m_h_electron_emins1;
  NTuple::Array<float> m_h_electron_wtots1;
  NTuple::Array<float> m_h_electron_fracs1;
  // for softe electrons
  NTuple::Array<float> m_h_electron_EtaCorrMag;
  NTuple::Array<float> m_h_electron_f1core;
  NTuple::Array<float> m_h_electron_f3core;
  NTuple::Array<float> m_h_electron_pos7;
  NTuple::Array<float> m_h_electron_iso;
  NTuple::Array<float> m_h_electron_widths2;
 
  // LArEM
  static const int max_lar_num=300;
  NTuple::Item<long>   m_h_larn;
  NTuple::Array<long>  m_h_larnsamples;
  NTuple::Array<float> m_h_lar_time;
  NTuple::Array<float> m_h_larEMB0E;
  NTuple::Array<float> m_h_larEMB1E;
  NTuple::Array<float> m_h_larEMB2E;
  NTuple::Array<float> m_h_larEMB3E;
  NTuple::Array<float> m_h_larEME0E;
  NTuple::Array<float> m_h_larEME1E;
  NTuple::Array<float> m_h_larEME2E;
  NTuple::Array<float> m_h_larEME3E;
  NTuple::Array<float> m_h_larEMB0Eta;
  NTuple::Array<float> m_h_larEMB1Eta;
  NTuple::Array<float> m_h_larEMB2Eta;
  NTuple::Array<float> m_h_larEMB3Eta;
  NTuple::Array<float> m_h_larEME0Eta;
  NTuple::Array<float> m_h_larEME1Eta;
  NTuple::Array<float> m_h_larEME2Eta;
  NTuple::Array<float> m_h_larEME3Eta;
  NTuple::Array<float> m_h_larEMB0Phi;
  NTuple::Array<float> m_h_larEMB1Phi;
  NTuple::Array<float> m_h_larEMB2Phi;
  NTuple::Array<float> m_h_larEMB3Phi;
  NTuple::Array<float> m_h_larEME0Phi;
  NTuple::Array<float> m_h_larEME1Phi;
  NTuple::Array<float> m_h_larEME2Phi;
  NTuple::Array<float> m_h_larEME3Phi;
  NTuple::Array<float> m_h_lareta;
  NTuple::Array<float> m_h_larphi;
  NTuple::Array<float> m_h_larenergy;
  NTuple::Array<long> m_h_larinBarrel;
  NTuple::Array<long> m_h_larinEndcap;
 
  // EMTopo
  static const int max_emtopo_num=300;
  NTuple::Item<long> m_h_emtopon;
  NTuple::Array<long> m_h_emtoponsamples;

  NTuple::Array<float> m_h_emtopo_time;
  NTuple::Array<float> m_h_emtopoeta;
  NTuple::Array<float> m_h_emtopophi;
  NTuple::Array<float> m_h_emtopoenergy;
  NTuple::Array<long> m_h_emtopoinBarrel;
  NTuple::Array<long> m_h_emtopoinEndcap;

  NTuple::Array<float> m_h_emtopo_first_phi;
  NTuple::Array<float> m_h_emtopo_first_eta;
  NTuple::Array<float> m_h_emtopo_second_r;
  NTuple::Array<float> m_h_emtopo_second_lambda;
  NTuple::Array<float> m_h_emtopo_center_lambda;
  //NTuple::Array<float> m_h_emtopo_first_eng_dens;
  NTuple::Array<float> m_h_emtopo_eng_frac_em;

  // CaloCalTopo (calibrated)
  static const int max_calocaltopo_num=5000;
  NTuple::Item<long> m_h_calocaltopon;
  NTuple::Array<long> m_h_calocaltoponsamples;
  NTuple::Array<float> m_h_calocaltopo_time;
  NTuple::Array<float> m_h_calocaltopoeta;
  NTuple::Array<float> m_h_calocaltopophi;
  NTuple::Array<float> m_h_calocaltopoenergy;
  NTuple::Array<long> m_h_calocaltopoinBarrel;
  NTuple::Array<long> m_h_calocaltopoinEndcap;

  NTuple::Array<float> m_h_calocaltopo_first_phi;
  NTuple::Array<float> m_h_calocaltopo_first_eta;
  NTuple::Array<float> m_h_calocaltopo_second_r;
  NTuple::Array<float> m_h_calocaltopo_second_lambda;
  NTuple::Array<float> m_h_calocaltopo_center_lambda;
  //NTuple::Array<float> m_h_calocaltopo_first_eng_dens;
  NTuple::Array<float> m_h_calocaltopo_eng_frac_em;

  // MUID Muons
  static const int max_muidmuon_num=300;
  NTuple::Item<long> m_h_muidmuon_num;
  // Muidmuon kinematics
  NTuple::Array<float> m_h_muidmuon_pt;
  NTuple::Array<float> m_h_muidmuon_eta;
  NTuple::Array<float> m_h_muidmuon_phi;
  NTuple::Array<float> m_h_muidmuon_e; 
  NTuple::Array<float> m_h_muidmuon_matchChi2OverDoF; 
  NTuple::Array<float> m_h_muidmuon_fitChi2OverDoF; 
  NTuple::Array<float> m_h_muidmuon_energyLoss_Deposit; 
  NTuple::Array<float> m_h_muidmuon_energyLoss_Sigma; 
  // author
  NTuple::Array<long> m_h_muidmuon_author;
  NTuple::Array<long> m_h_muidmuon_isCombinedMuon;
  NTuple::Array<long> m_h_muidmuon_isStandAloneMuon;
  NTuple::Array<long> m_h_muidmuon_isLowPtReconstructedMuon;
  NTuple::Array<long> m_h_muidmuon_hasCombinedMuon;
  NTuple::Array<long> m_h_muidmuon_hasInDetTrackParticle;
  NTuple::Array<long> m_h_muidmuon_hasMuonSpectrometerTrackParticle;
  NTuple::Array<long> m_h_muidmuon_hasMuonExtrapolatedTrackParticle;
  NTuple::Array<long> m_h_muidmuon_hasCombinedMuonTrackParticle;
  NTuple::Array<long> m_h_muidmuon_hasCluster;
  NTuple::Array<long> m_h_muidmuon_isHighPt; 
  NTuple::Array<long> m_h_muidmuon_isLowPt; 
  // parameters
  NTuple::Array<float> m_h_muidmuon_etcone10; 
  NTuple::Array<float> m_h_muidmuon_etcone20; 
  NTuple::Array<float> m_h_muidmuon_etcone30; 
  NTuple::Array<float> m_h_muidmuon_etcone40; 
  NTuple::Array<float> m_h_muidmuon_etcone; 
  NTuple::Array<float> m_h_muidmuon_etcone50; 
  NTuple::Array<float> m_h_muidmuon_etcone60; 
  NTuple::Array<float> m_h_muidmuon_etcone70; 
  NTuple::Array<float> m_h_muidmuon_nucone10;
  NTuple::Array<float> m_h_muidmuon_nucone20; 
  NTuple::Array<float> m_h_muidmuon_nucone30; 
  NTuple::Array<float> m_h_muidmuon_nucone40; 
  NTuple::Array<float> m_h_muidmuon_nucone; 
  NTuple::Array<float> m_h_muidmuon_nucone50; 
  NTuple::Array<float> m_h_muidmuon_nucone60; 
  NTuple::Array<float> m_h_muidmuon_nucone70; 
  //                                            
  NTuple::Array<long> m_h_muidmuon_numberOfBLayerHits;
  NTuple::Array<long> m_h_muidmuon_numberOfPixelHits;
  NTuple::Array<long> m_h_muidmuon_numberOfSCTHits;
  NTuple::Array<long> m_h_muidmuon_numberOfTRTHits;
  NTuple::Array<long> m_h_muidmuon_numberOfTRTHighThresholdHits;
  NTuple::Array<long> m_h_muidmuon_numberOfMDTHits;
  NTuple::Array<long> m_h_muidmuon_numberOfCSCEtaHits;
  NTuple::Array<long> m_h_muidmuon_numberOfCSCPhiHits;
  NTuple::Array<long> m_h_muidmuon_numberOfRPCEtaHits;
  NTuple::Array<long> m_h_muidmuon_numberOfRPCPhiHits;
  NTuple::Array<long> m_h_muidmuon_numberOfTGCEtaHits;
  NTuple::Array<long> m_h_muidmuon_numberOfTGCPhiHits;
  // track info (only the default track)
  NTuple::Array<float> m_h_muidmuon_track_pt;
  NTuple::Array<float> m_h_muidmuon_track_eta;
  NTuple::Array<float> m_h_muidmuon_track_phi;
  NTuple::Array<float> m_h_muidmuon_track_e;
  NTuple::Array<float> m_h_muidmuon_track_d0;
  NTuple::Array<float> m_h_muidmuon_track_z0;   
  NTuple::Array<float> m_h_muidmuon_track_charge;

  // Staco Muons
  static const int max_stacomuon_num=300;
  NTuple::Item<long> m_h_stacomuon_num;
  // Stacomuon kinematics
  NTuple::Array<float> m_h_stacomuon_pt;
  NTuple::Array<float> m_h_stacomuon_eta;
  NTuple::Array<float> m_h_stacomuon_phi;
  NTuple::Array<float> m_h_stacomuon_e; 
  NTuple::Array<float> m_h_stacomuon_matchChi2OverDoF; 
  NTuple::Array<float> m_h_stacomuon_fitChi2OverDoF; 
  NTuple::Array<float> m_h_stacomuon_energyLoss_Deposit; 
  NTuple::Array<float> m_h_stacomuon_energyLoss_Sigma; 
  // author
  NTuple::Array<long> m_h_stacomuon_author;
  NTuple::Array<long> m_h_stacomuon_isCombinedMuon;
  NTuple::Array<long> m_h_stacomuon_isStandAloneMuon;
  NTuple::Array<long> m_h_stacomuon_isLowPtReconstructedMuon;
  NTuple::Array<long> m_h_stacomuon_hasCombinedMuon;
  NTuple::Array<long> m_h_stacomuon_hasInDetTrackParticle;
  NTuple::Array<long> m_h_stacomuon_hasMuonSpectrometerTrackParticle;
  NTuple::Array<long> m_h_stacomuon_hasMuonExtrapolatedTrackParticle;
  NTuple::Array<long> m_h_stacomuon_hasCombinedMuonTrackParticle;
  NTuple::Array<long> m_h_stacomuon_hasCluster;
  NTuple::Array<long> m_h_stacomuon_isHighPt; 
  NTuple::Array<long> m_h_stacomuon_isLowPt; 
  // parameters
  NTuple::Array<float> m_h_stacomuon_etcone10; 
  NTuple::Array<float> m_h_stacomuon_etcone20; 
  NTuple::Array<float> m_h_stacomuon_etcone30; 
  NTuple::Array<float> m_h_stacomuon_etcone40; 
  NTuple::Array<float> m_h_stacomuon_etcone; 
  NTuple::Array<float> m_h_stacomuon_etcone50; 
  NTuple::Array<float> m_h_stacomuon_etcone60; 
  NTuple::Array<float> m_h_stacomuon_etcone70; 
  NTuple::Array<float> m_h_stacomuon_nucone10;
  NTuple::Array<float> m_h_stacomuon_nucone20; 
  NTuple::Array<float> m_h_stacomuon_nucone30; 
  NTuple::Array<float> m_h_stacomuon_nucone40; 
  NTuple::Array<float> m_h_stacomuon_nucone; 
  NTuple::Array<float> m_h_stacomuon_nucone50; 
  NTuple::Array<float> m_h_stacomuon_nucone60; 
  NTuple::Array<float> m_h_stacomuon_nucone70; 
  //                                            
  NTuple::Array<long> m_h_stacomuon_numberOfBLayerHits;
  NTuple::Array<long> m_h_stacomuon_numberOfPixelHits;
  NTuple::Array<long> m_h_stacomuon_numberOfSCTHits;
  NTuple::Array<long> m_h_stacomuon_numberOfTRTHits;
  NTuple::Array<long> m_h_stacomuon_numberOfTRTHighThresholdHits;
  NTuple::Array<long> m_h_stacomuon_numberOfMDTHits;
  NTuple::Array<long> m_h_stacomuon_numberOfCSCEtaHits;
  NTuple::Array<long> m_h_stacomuon_numberOfCSCPhiHits;
  NTuple::Array<long> m_h_stacomuon_numberOfRPCEtaHits;
  NTuple::Array<long> m_h_stacomuon_numberOfRPCPhiHits;
  NTuple::Array<long> m_h_stacomuon_numberOfTGCEtaHits;
  NTuple::Array<long> m_h_stacomuon_numberOfTGCPhiHits;
  // track info (only the default track)
  NTuple::Array<float> m_h_stacomuon_track_pt;
  NTuple::Array<float> m_h_stacomuon_track_eta;
  NTuple::Array<float> m_h_stacomuon_track_phi;
  NTuple::Array<float> m_h_stacomuon_track_e;
  NTuple::Array<float> m_h_stacomuon_track_d0;
  NTuple::Array<float> m_h_stacomuon_track_z0;   
  NTuple::Array<float> m_h_stacomuon_track_charge;

  // Photon
  static const int max_photon_num=300;
  NTuple::Item<long> m_h_photon_num;
  // Photon kinematics
  NTuple::Array<float> m_h_photon_pt;
  NTuple::Array<float> m_h_photon_eta;
  NTuple::Array<float> m_h_photon_phi;
  NTuple::Array<float> m_h_photon_e; 
  // Photon algorithm
  NTuple::Array<long>  m_h_photon_egamma;        
  //  NTuple::Array<long>  m_h_photon_softe;        
  // EgammaPID parameters
  NTuple::Array<float> m_h_photon_IsEM;
  NTuple::Array<float> m_h_photon_ElectronWeight;     
  NTuple::Array<float> m_h_photon_BgWeight;     
  NTuple::Array<float> m_h_photon_NeuralNet;     
  NTuple::Array<float> m_h_photon_SofteIsEM;
  NTuple::Array<float> m_h_photon_SofteElectronWeight;     
  NTuple::Array<float> m_h_photon_SofteBgWeight;     
  NTuple::Array<float> m_h_photon_SofteNeuralNet;     
  // Photon Parameters
  NTuple::Array<float> m_h_photon_etcone;  
  NTuple::Array<float> m_h_photon_etcone20;
  NTuple::Array<float> m_h_photon_etcone30;
  NTuple::Array<float> m_h_photon_etcone40; 
  NTuple::Array<float> m_h_photon_etaBE2;
  NTuple::Array<float> m_h_photon_et37;
  NTuple::Array<float> m_h_photon_e233;
  NTuple::Array<float> m_h_photon_e237;
  NTuple::Array<float> m_h_photon_e277;
  NTuple::Array<float> m_h_photon_ethad1;
  NTuple::Array<float> m_h_photon_weta1;
  NTuple::Array<float> m_h_photon_weta2;
  NTuple::Array<float> m_h_photon_f1;
  NTuple::Array<float> m_h_photon_e2tsts1;
  NTuple::Array<float> m_h_photon_emins1;
  NTuple::Array<float> m_h_photon_wtots1;
  NTuple::Array<float> m_h_photon_fracs1;
  // Photon conversion info
  NTuple::Array<long> m_h_photon_conversionAngleMatch;
  NTuple::Array<long> m_h_photon_conversionTrackMatch;

  NTuple::Array<float> m_h_photon_conversionVx_x;
  NTuple::Array<float> m_h_photon_conversionVx_y;
  NTuple::Array<float> m_h_photon_conversionVx_z;

  NTuple::Array<float> m_h_photon_conversiontrack1_pt;
  NTuple::Array<float> m_h_photon_conversiontrack1_eta;
  NTuple::Array<float> m_h_photon_conversiontrack1_phi;
  NTuple::Array<float> m_h_photon_conversiontrack1_e;
  NTuple::Array<float> m_h_photon_conversiontrack1_charge;

  NTuple::Array<float> m_h_photon_conversiontrack2_pt;
  NTuple::Array<float> m_h_photon_conversiontrack2_eta;
  NTuple::Array<float> m_h_photon_conversiontrack2_phi;
  NTuple::Array<float> m_h_photon_conversiontrack2_e;
  NTuple::Array<float> m_h_photon_conversiontrack2_charge;
 
  // CTP_Decision
  NTuple::Item<long> m_h_ctpdecision_word0;
  NTuple::Item<long> m_h_ctpdecision_word1;
  NTuple::Item<long> m_h_ctpdecision_word2;

  // L1TriggerDecision
  NTuple::Item<long> m_h_passed_L1;
  NTuple::Item<long> m_h_passed_L2;
  NTuple::Item<long> m_h_passed_EF;
  // L1 for 12.03.31 data
  //  NTuple::Item<long> m_h_L1_EM01;
  //  NTuple::Item<long> m_h_L1_2EM15I;
  //  NTuple::Item<long> m_h_L1_EM25I;
  //  NTuple::Item<long> m_h_L1_EM60 ;
  //  NTuple::Item<long> m_h_L1_MU06 ;
  //  NTuple::Item<long> m_h_L1_2MU06;
  //  NTuple::Item<long> m_h_L1_MU08 ;
  //  NTuple::Item<long> m_h_L1_MU10 ;
  //  NTuple::Item<long> m_h_L1_MU11 ;
  //  NTuple::Item<long> m_h_L1_MU20 ;
  //  NTuple::Item<long> m_h_L1_MU40 ;
  //  NTuple::Item<long> m_h_L1_HA01;
  //  NTuple::Item<long> m_h_L1_HA07;
  //  NTuple::Item<long> m_h_L1_HA10I;
  //  NTuple::Item<long> m_h_L1_HA15I;
  //  NTuple::Item<long> m_h_L1_HA35I;

  // L1 for 12.04 data
  NTuple::Item<long> m_h_L1_2EM15;
  NTuple::Item<long> m_h_L1_2EM15I;
  NTuple::Item<long> m_h_L1_EM25;
  NTuple::Item<long> m_h_L1_EM25I;
  NTuple::Item<long> m_h_L1_EM5;
  NTuple::Item<long> m_h_L1_EM60 ;
  NTuple::Item<long> m_h_L1_TAU05;
  NTuple::Item<long> m_h_L1_TAU10 ;
  NTuple::Item<long> m_h_L1_TAU10I ;
  NTuple::Item<long> m_h_L1_TAU15 ;
  NTuple::Item<long> m_h_L1_TAU15I;
  NTuple::Item<long> m_h_L1_TAU20I;
  NTuple::Item<long> m_h_L1_TAU25I;
  NTuple::Item<long> m_h_L1_TAU35I ;
  NTuple::Item<long> m_h_L1_XE20;
  NTuple::Item<long> m_h_L1_XE30;
  NTuple::Item<long> m_h_L1_XE40 ;
  NTuple::Item<long> m_h_L1_XE50;
  NTuple::Item<long> m_h_L1_XE100;
  NTuple::Item<long> m_h_L1_XE200;
  NTuple::Item<long> m_h_L1_MU06 ;
  NTuple::Item<long> m_h_L1_2MU06;
  NTuple::Item<long> m_h_L1_MU08 ;
  NTuple::Item<long> m_h_L1_MU10 ;
  NTuple::Item<long> m_h_L1_MU11 ;
  NTuple::Item<long> m_h_L1_MU20 ;
  NTuple::Item<long> m_h_L1_MU40 ;
  NTuple::Item<long> m_h_L1_J35 ;
  NTuple::Item<long> m_h_L1_J45 ;
  NTuple::Item<long> m_h_L1_2J45 ;
  NTuple::Item<long> m_h_L1_3J45 ;
  NTuple::Item<long> m_h_L1_4J45 ;
  NTuple::Item<long> m_h_L1_FJ30 ;
  NTuple::Item<long> m_h_L1_BJT15;
 
 
  NTuple::Item<long> m_h_L2_e10;
  NTuple::Item<long> m_h_L2_e10TRTxK;
  NTuple::Item<long> m_h_L2_e10L2_e10;
  NTuple::Item<long> m_h_L2_e15iL2_e15i;
  NTuple::Item<long> m_h_L2_e25i;
  NTuple::Item<long> m_h_L2_e60;
  NTuple::Item<long> m_h_L2_g10;
  NTuple::Item<long> m_h_L2_g20iL2_g20i;
  NTuple::Item<long> m_h_L2_g60;
  NTuple::Item<long> m_h_L2_Ze10e10;
  NTuple::Item<long> m_h_L2_tau10;
  NTuple::Item<long> m_h_L2_tau10i;
  NTuple::Item<long> m_h_L2_tau15;
  NTuple::Item<long> m_h_L2_tau15i;
  NTuple::Item<long> m_h_L2_tau20i;
  NTuple::Item<long> m_h_L2_tau25i;
  NTuple::Item<long> m_h_L2_tau35i;
  NTuple::Item<long> m_h_L2_tauNoCut;
  NTuple::Item<long> m_h_L2_mu6;
  NTuple::Item<long> m_h_L2_mu6l;
  NTuple::Item<long> m_h_L2_mu20i;
  NTuple::Item<long> m_h_L2_jet20kt;
  NTuple::Item<long> m_h_L2_jet20;
  NTuple::Item<long> m_h_L2_jet400;
  NTuple::Item<long> m_h_L2_jet350L2_jet350;
  NTuple::Item<long> m_h_L2_jet165L2_jet165L2_jet165; 	
  NTuple::Item<long> m_h_L2_jet110L2_jet110L2_jet110L2_jet110; 
  

  NTuple::Item<long> m_h_EF_MuonTRTExt_mu6l;
  NTuple::Item<long> m_h_EF_e10;
  NTuple::Item<long> m_h_EF_e10TRTxK;
  NTuple::Item<long> m_h_EF_e15iEF_e15i;
  NTuple::Item<long> m_h_EF_e25i;
  NTuple::Item<long> m_h_EF_e60;
  NTuple::Item<long> m_h_EF_g10;
  NTuple::Item<long> m_h_EF_g20iEF_g20i;
  NTuple::Item<long> m_h_EF_g60;
  NTuple::Item<long> m_h_EF_jet110EF_jet110EF_jet110EF_jet110;
  NTuple::Item<long> m_h_EF_jet165EF_jet165EF_jet165;
  NTuple::Item<long> m_h_EF_jet20;
  NTuple::Item<long> m_h_EF_jet20kt;
  NTuple::Item<long> m_h_EF_jet350EF_jet350;
  NTuple::Item<long> m_h_EF_jet400;
  NTuple::Item<long> m_h_EF_mu20i;
  NTuple::Item<long> m_h_EF_mu6;
  NTuple::Item<long> m_h_EF_mu6l;
  NTuple::Item<long> m_h_EF_tau10;
  NTuple::Item<long> m_h_EF_tau10i;
  NTuple::Item<long> m_h_EF_tau15;
  NTuple::Item<long> m_h_EF_tau15i;
  NTuple::Item<long> m_h_EF_tau20i;
  NTuple::Item<long> m_h_EF_tau25i;
  NTuple::Item<long> m_h_EF_tau35i;
  NTuple::Item<long> m_h_EF_tauNoCut;

  //// Event Filter egamma
  egammaqpoint*             m_egammaqpoint;
  egammaqgcld*              m_egammaqgcld;
  static const int max_egamma_num=300;
  NTuple::Item<long>   m_h_egamma_num;           // number of reconstructed seeds (clusters or tracks)
  NTuple::Array<float> m_h_egamma_e;             // E of electron/photon
  NTuple::Array<float> m_h_egamma_eta;           // eta of electron/photon
  NTuple::Array<float> m_h_egamma_phi;           // phi of electron/photon
  NTuple::Array<long>  m_h_egamma_isem;          // e-ID flag, 0:e, >=1:jet
  NTuple::Array<float> m_h_egamma_epiANN;        // neural net output, ~0 => pi, ~1 => e
  NTuple::Array<double> m_h_egamma_EMWeight;     // EMWeight PDF
  NTuple::Array<double> m_h_egamma_PionWeight;   // PionWeight PDF
  NTuple::Array<long>  m_h_egamma_isemse;        // e-ID flag, 0:e, >=1:jet
  NTuple::Array<float> m_h_egamma_epiANNse;      // neural net output Softe, ~0 => pi, ~1 => e
  NTuple::Array<double> m_h_egamma_EMWeightse;   // EMWeight PDF SOfte
  NTuple::Array<double> m_h_egamma_PionWeightse; // PionWeight PDF Softe

  NTuple::Array<float> m_h_egamma_cl_et;    // ET of cluster
  NTuple::Array<float> m_h_egamma_cl_eta;   // eta of cluster
  NTuple::Array<float> m_h_egamma_cl_phi;   // phi of cluster
  NTuple::Array<float> m_h_egamma_e0;       // energy in presampler
  NTuple::Array<float> m_h_egamma_e1;       // energy in em sampling 1
  NTuple::Array<float> m_h_egamma_e2;       // energy in em sampling 2
  NTuple::Array<float> m_h_egamma_e3;       // energy in em sampling 3
  NTuple::Array<float> m_h_egamma_eta0;     // eta in presampler 
  NTuple::Array<float> m_h_egamma_eta1;     // eta in em sampling 1 
  NTuple::Array<float> m_h_egamma_eta2;     // eta in em sampling 2 
  NTuple::Array<float> m_h_egamma_eta3;     // eta in em sampling 3 
  NTuple::Array<float> m_h_egamma_phi0;     // phi in presampler 
  NTuple::Array<float> m_h_egamma_phi1;     // phi in em sampling 1 
  NTuple::Array<float> m_h_egamma_phi2;     // phi in em sampling 2 
  NTuple::Array<float> m_h_egamma_phi3;     // phi in em sampling 3 

  NTuple::Array<float> m_h_egamma_etap;     // eta calculated from pointing
  NTuple::Array<float> m_h_egamma_zvertex;  // z-vertex position
  NTuple::Array<float> m_h_egamma_errz ;    // error on z-vertex
  NTuple::Array<float> m_h_egamma_depth ;   // shower depth
  
  NTuple::Array<float> m_h_egamma_ethad1;   // ET leakage into sampling 1 of had calo
  NTuple::Array<float> m_h_egamma_ethad;    // ET leakage into had calo
  NTuple::Array<float> m_h_egamma_ehad1;    // E leakage into 1st sampling of had calo
  NTuple::Array<float> m_h_egamma_e011;     // uncor. energy in 1x1 cells in pre sampler
  NTuple::Array<float> m_h_egamma_e131;     // uncor. energy in 3x1 cells in em strip
  NTuple::Array<float> m_h_egamma_f1;       // fraction of energy found in em Samp 1
  NTuple::Array<float> m_h_egamma_f3;       // fraction of energy found in em Samp 3
  NTuple::Array<float> m_h_egamma_f1core;   // fraction of energy found in em Samp 1
  NTuple::Array<float> m_h_egamma_f3core;   // fraction of energy found in em Samp 3
  NTuple::Array<float> m_h_egamma_e233;     // uncor. energy in 3x3 cells in em samp 2
  NTuple::Array<float> m_h_egamma_e235;     // uncor. energy in 3x5 cells in em samp 2
  NTuple::Array<float> m_h_egamma_e237;     // uncor. energy in 3x7 cells in em samp 2
  NTuple::Array<float> m_h_egamma_e277;     // uncor. energy in 7x7 cells in em samp 2
  NTuple::Array<float> m_h_egamma_e335;     // uncor. energy in 3x5 cells in em samp 3
  NTuple::Array<float> m_h_egamma_weta1;    // cor. width in 3 strips in the 1st samp.
  NTuple::Array<float> m_h_egamma_weta2;    // cor. width in 3x5 cells in the 2nd samp
  NTuple::Array<float> m_h_egamma_e2ts1;    // 2nd max in strips calc. in 3 strips 
  NTuple::Array<float> m_h_egamma_e2tsts1;  // 2nd maximum in strips
  NTuple::Array<float> m_h_egamma_fracs1;   // E outside core (E(+-7)-E(+-3))/E(+-7)
  NTuple::Array<float> m_h_egamma_widths1;  // uncor width in em samp 1 in 3 strips
  NTuple::Array<float> m_h_egamma_widths2;  // uncor width in em samp 2 in 3x5 cells
  NTuple::Array<float> m_h_egamma_poscs1;   // position within cell in 1st sampling
  NTuple::Array<float> m_h_egamma_poscs2;   // position within cell in 2nd sampling
  NTuple::Array<float> m_h_egamma_asy1;     // uncor asymmetry in em samp 1 in 3 strips
  NTuple::Array<float> m_h_egamma_pos7;     // diff +/- 7 cells
  NTuple::Array<float> m_h_egamma_isol;     // 1 - E3x7 / E7x7
  NTuple::Array<float> m_h_egamma_barys1;   // barycentre in samp 1 using 3 strips
  NTuple::Array<float> m_h_egamma_wtots1;   // total width in em samp 1 in 20 strips
  NTuple::Array<float> m_h_egamma_emins1;   // E of strip with min between max 1 & 2
  NTuple::Array<float> m_h_egamma_emaxs1;   // E of strip with maximal energy deposit
  NTuple::Array<float> m_h_egamma_ecore;    // Core energy in em calo   
  NTuple::Array<float> m_h_egamma_etcone;   // Energy in cone of 0.45
  NTuple::Array<float> m_h_egamma_etcone20; // Energy in cone of 0.20
  NTuple::Array<float> m_h_egamma_etcone30; // Energy in cone of 0.30
  NTuple::Array<float> m_h_egamma_etcone40; // Energy in cone of 0.40

  // ntuple variable for TrackParticle
  NTuple::Array<long>  m_h_egamma_primary;  // 1 = matched track is primary, 2 = secondary   
  NTuple::Array<long>  m_h_egamma_numtracks;// number of primary tracks in given window
  NTuple::Array<float> m_h_egamma_eoverp;   // energy match, E(calo)/p(track) 
  NTuple::Array<float> m_h_egamma_etacorr;  // eta(track) corrected for magnetic field 
  NTuple::Array<float> m_h_egamma_deta0;    // delta eta of trk extrapolated to calo S0
  NTuple::Array<float> m_h_egamma_dphi0;    // delta phi of trk extrapolated to calo S0
  NTuple::Array<float> m_h_egamma_deta1;    // delta eta of trk extrapolated to calo S1
  NTuple::Array<float> m_h_egamma_dphi1;    // delta phi of trk extrapolated to calo S1
  NTuple::Array<float> m_h_egamma_deta2;    // delta eta of trk extrapolated to calo S2
  NTuple::Array<float> m_h_egamma_dphi2;    // delta phi of trk extrapolated to calo S2
  NTuple::Array<float> m_h_egamma_deta3;    // delta eta of trk extrapolated to calo S3
  NTuple::Array<float> m_h_egamma_dphi3;    // delta phi of trk extrapolated to calo S3

  // TRT information from tracksummary
  NTuple::Array<long>  m_h_egamma_tracksummary_nTRThits;    // Number of TRT hits
  NTuple::Array<long>  m_h_egamma_tracksummary_nTRThitsHT;  // Number of TRT High Threshold hits

  // TrigElectron
  static const int max_trigelectron_num=300;
  NTuple::Item<long>   m_h_trigelectron_num;
  NTuple::Array<long>  m_h_trigelectron_container;
  NTuple::Array<float> m_h_trigelectron_pt;
  NTuple::Array<float> m_h_trigelectron_eta;
  NTuple::Array<float> m_h_trigelectron_phi;
  NTuple::Array<float> m_h_trigelectron_m;	    
  NTuple::Array<long>  m_h_trigelectron_isvalid;    
  NTuple::Array<long>  m_h_trigelectron_roiID;	    
  NTuple::Array<float> m_h_trigelectron_zvtx;	    
  NTuple::Array<float> m_h_trigelectron_err_eta;    
  NTuple::Array<float> m_h_trigelectron_err_phi;    
  NTuple::Array<float> m_h_trigelectron_err_zvtx;   
  NTuple::Array<float> m_h_trigelectron_err_pt;	    
  NTuple::Array<float> m_h_trigelectron_trkclusdeta;
  NTuple::Array<float> m_h_trigelectron_trkclusdphi;
  
  //new variables
  NTuple::Array<long>   m_h_trigelectron_track_algoID; 
  NTuple::Array<float>  m_h_trigelectron_track_chi2;
  NTuple::Array<long>   m_h_trigelectron_track_NStrawHits; 
  NTuple::Array<long>   m_h_trigelectron_track_NStraw; 
  NTuple::Array<long>   m_h_trigelectron_track_NStrawTime; 
  NTuple::Array<long>   m_h_trigelectron_track_NTRHits;
  NTuple::Array<float>  m_h_trigelectron_track_param_a0;
  NTuple::Array<float>  m_h_trigelectron_track_param_z0;
  NTuple::Array<float>  m_h_trigelectron_track_param_phi0;
  NTuple::Array<float>  m_h_trigelectron_track_param_eta;
  NTuple::Array<float>  m_h_trigelectron_track_param_pt;
  NTuple::Array<float>  m_h_trigelectron_track_param_ea0;
  NTuple::Array<float>  m_h_trigelectron_track_param_ez0;
  NTuple::Array<float>  m_h_trigelectron_track_param_ephi0;
  NTuple::Array<float>  m_h_trigelectron_track_param_eeta;
  NTuple::Array<float>  m_h_trigelectron_track_param_ept;
  NTuple::Array<float>  m_h_trigelectron_track_endparam_a0;
  NTuple::Array<float>  m_h_trigelectron_track_endparam_z0;
  NTuple::Array<float>  m_h_trigelectron_track_endparam_phi0;
  NTuple::Array<float>  m_h_trigelectron_track_endparam_eta;
  NTuple::Array<float>  m_h_trigelectron_track_endparam_pt;
  NTuple::Array<float>  m_h_trigelectron_track_endparam_ea0;
  NTuple::Array<float>  m_h_trigelectron_track_endparam_ez0;
  NTuple::Array<float>  m_h_trigelectron_track_endparam_ephi0;
  NTuple::Array<float>  m_h_trigelectron_track_endparam_eeta;
  NTuple::Array<float>  m_h_trigelectron_track_endparam_ept;
 
  NTuple::Array<float>  m_h_trigelectron_cluster_e;
  NTuple::Array<float>  m_h_trigelectron_cluster_et;
  NTuple::Array<float>  m_h_trigelectron_cluster_eta;
  NTuple::Array<float>  m_h_trigelectron_cluster_phi;
  NTuple::Array<float>  m_h_trigelectron_cluster_e237;
  NTuple::Array<float>  m_h_trigelectron_cluster_e277;
  NTuple::Array<float>  m_h_trigelectron_cluster_e2tsts1;
  NTuple::Array<float>  m_h_trigelectron_cluster_ehad1;
  NTuple::Array<float>  m_h_trigelectron_cluster_emaxs1;
  NTuple::Array<float>  m_h_trigelectron_cluster_fracs1;
  NTuple::Array<float>  m_h_trigelectron_cluster_weta2;

  // L1EMTauObject
  static const int max_l1emtauobject_num=300;
  NTuple::Item<long> m_h_l1emtauobject_num;
  NTuple::Array<float> m_h_l1emtauobject_emcore;
  NTuple::Array<float> m_h_l1emtauobject_hdcore;
  NTuple::Array<float> m_h_l1emtauobject_emclus;
  NTuple::Array<float> m_h_l1emtauobject_tauclus;
  NTuple::Array<float> m_h_l1emtauobject_emisol;
  NTuple::Array<float> m_h_l1emtauobject_hdisol;
  NTuple::Array<float> m_h_l1emtauobject_eta;
  NTuple::Array<float> m_h_l1emtauobject_phi;
  
  // L1JetObject
  static const int max_l1jetobject_num=300;
  NTuple::Item<long> m_h_l1jetobject_num;
  NTuple::Array<float> m_h_l1jetobject_L1Jet_ET4x4;
  NTuple::Array<float> m_h_l1jetobject_L1Jet_ET6x6;
  NTuple::Array<float> m_h_l1jetobject_L1Jet_ET8x8;
  NTuple::Array<float> m_h_l1jetobject_L1Jet_eta;
  NTuple::Array<float> m_h_l1jetobject_L1Jet_phi;

  // L1EtmissObject
  NTuple::Item<float> m_h_l1etmissobject_L1ETM_Ex;
  NTuple::Item<float> m_h_l1etmissobject_L1ETM_Ey;
  NTuple::Item<float> m_h_l1etmissobject_L1ETM_EtMiss;
  NTuple::Item<float> m_h_l1etmissobject_L1ETM_EtSum;

  // TauJets 
  static const int max_taujet_num=100;
  NTuple::Item<long> m_h_taujet_num; 
  NTuple::Array<float> m_h_taujet_pt;
  NTuple::Array<float> m_h_taujet_eta;
  NTuple::Array<float> m_h_taujet_phi;
  NTuple::Array<float> m_h_taujet_e;
  // taujet track info 
  NTuple::Array<long>  m_h_taujet_numTrack;
  NTuple::Array<float> m_h_taujet_author; 	
  // TauJet common parameters
  NTuple::Array<float> m_h_taujet_etHadCalib; 	
  NTuple::Array<float> m_h_taujet_etEMCalib; 	
  NTuple::Array<float> m_h_taujet_emRadius; 	
  NTuple::Array<float> m_h_taujet_isolationFraction; 	
  NTuple::Array<float> m_h_taujet_centralityFraction; 	
  NTuple::Array<float> m_h_taujet_stripWidth2; 	
  NTuple::Array<float> m_h_taujet_nStripCells;	
  // TauRec parameters
  NTuple::Array<float> m_h_taujet_logLikelihoodRatio; 	
  NTuple::Array<float> m_h_taujet_lowPtTauJetDiscriminant; 	
  NTuple::Array<float> m_h_taujet_lowPtTauEleDiscriminant; 	
  NTuple::Array<float> m_h_taujet_tauJetNeuralnetwork;
  NTuple::Array<float> m_h_taujet_tauENeuralNetwork;
  // Tau1p3p parameters
  NTuple::Array<float> m_h_taujet_annularIsolationFraction;	
  NTuple::Array<float> m_h_taujet_etCaloAtEMScale;
  NTuple::Array<float> m_h_taujet_etChargedHadCells;
  NTuple::Array<float> m_h_taujet_etOtherEMCells;
  NTuple::Array<float> m_h_taujet_etOtherHadCells;
  NTuple::Array<float> m_h_taujet_discriminant;
  NTuple::Array<float> m_h_taujet_discPDERS;	
  NTuple::Array<float> m_h_taujet_discNN;
  NTuple::Array<float> m_h_taujet_sumPTtracks;	
  NTuple::Array<float> m_h_taujet_nAssocTracksCore;	
  NTuple::Array<float> m_h_taujet_nAssocTracksIsol;	
  NTuple::Array<float> m_h_taujet_rWidth2Trk3P; 	
  NTuple::Array<float> m_h_taujet_massTrk3P;	
  NTuple::Array<float> m_h_taujet_signDOTrk3P;


  // TauJet1p3p
  static const int max_taujet1p3p_num=100;
  NTuple::Item<long> m_h_taujet1p3p_num; 
  NTuple::Array<float> m_h_taujet1p3p_pt;
  NTuple::Array<float> m_h_taujet1p3p_eta;
  NTuple::Array<float> m_h_taujet1p3p_phi;
  NTuple::Array<float> m_h_taujet1p3p_e;
  // taujet track info 
  NTuple::Array<long>  m_h_taujet1p3p_numTrack;
  NTuple::Array<float> m_h_taujet1p3p_author; 	
  // Taujet common parameters
  NTuple::Array<float> m_h_taujet1p3p_etHadCalib; 	
  NTuple::Array<float> m_h_taujet1p3p_etEMCalib; 	
  NTuple::Array<float> m_h_taujet1p3p_emRadius; 	
  NTuple::Array<float> m_h_taujet1p3p_isolationFraction; 	
  NTuple::Array<float> m_h_taujet1p3p_centralityFraction; 	
  NTuple::Array<float> m_h_taujet1p3p_stripWidth2; 	
  NTuple::Array<float> m_h_taujet1p3p_nStripCells;	
  // TauRec parameters
  NTuple::Array<float> m_h_taujet1p3p_logLikelihoodRatio; 	
  NTuple::Array<float> m_h_taujet1p3p_lowPtTauJetDiscriminant; 	
  NTuple::Array<float> m_h_taujet1p3p_lowPtTauEleDiscriminant; 	
  NTuple::Array<float> m_h_taujet1p3p_tauJetNeuralnetwork;
  NTuple::Array<float> m_h_taujet1p3p_tauENeuralNetwork;
  // Tau1p3p parameters
  NTuple::Array<float> m_h_taujet1p3p_annularIsolationFraction;	
  NTuple::Array<float> m_h_taujet1p3p_etCaloAtEMScale;
  NTuple::Array<float> m_h_taujet1p3p_etChargedHadCells;
  NTuple::Array<float> m_h_taujet1p3p_etOtherEMCells;
  NTuple::Array<float> m_h_taujet1p3p_etOtherHadCells;
  NTuple::Array<float> m_h_taujet1p3p_discriminant;
  NTuple::Array<float> m_h_taujet1p3p_discPDERS;	
  NTuple::Array<float> m_h_taujet1p3p_discNN;
  NTuple::Array<float> m_h_taujet1p3p_sumPTtracks;	
  NTuple::Array<float> m_h_taujet1p3p_nAssocTracksCore;	
  NTuple::Array<float> m_h_taujet1p3p_nAssocTracksIsol;	
  NTuple::Array<float> m_h_taujet1p3p_rWidth2Trk3P; 	
  NTuple::Array<float> m_h_taujet1p3p_massTrk3P;	
  NTuple::Array<float> m_h_taujet1p3p_signDOTrk3P;
  



  // Truth particles (Generator truth only)
  static const int max_truth_num=10000;
  NTuple::Item<long>   m_h_truth_num; 
  NTuple::Array<float> m_h_truth_pt;
  NTuple::Array<float> m_h_truth_eta;
  NTuple::Array<float> m_h_truth_phi;
  NTuple::Array<float> m_h_truth_e;
  NTuple::Array<float> m_h_truth_pdgId;
  NTuple::Array<float> m_h_truth_Barcode;
  NTuple::Array<float> m_h_truth_Status;
  NTuple::Array<float> m_h_truth_etisolation_cone10;
  NTuple::Array<float> m_h_truth_etisolation_cone20;
  NTuple::Array<float> m_h_truth_etisolation_cone30;
  NTuple::Array<float> m_h_truth_etisolation_cone40;
  NTuple::Array<float> m_h_truth_etisolation_cone45;
  NTuple::Array<float> m_h_truth_etisolation_cone50;
  NTuple::Array<float> m_h_truth_etisolation_cone60;
  NTuple::Array<float> m_h_truth_etisolation_cone70;
//  NTuple::Array<float> m_h_truth_NbrOfCones;
  /// Production vertex for each true particle
  NTuple::Array<long>  m_h_truth_hasProductionVertex;
  NTuple::Array<float> m_h_truth_productionVertex_x;
  NTuple::Array<float> m_h_truth_productionVertex_y;
  NTuple::Array<float> m_h_truth_productionVertex_z;
  NTuple::Array<float> m_h_truth_productionVertex_t;
  NTuple::Array<float> m_h_truth_productionVertex_perp;
  NTuple::Array<float> m_h_truth_productionVertex_eta;
  NTuple::Array<float> m_h_truth_productionVertex_phi;
  NTuple::Array<float> m_h_truth_productionVertex_r;
  /// Decay vertex for each true particle
  NTuple::Array<long>  m_h_truth_hasDecayVertex;
  NTuple::Array<float> m_h_truth_decayVertex_x;
  NTuple::Array<float> m_h_truth_decayVertex_y;
  NTuple::Array<float> m_h_truth_decayVertex_z;
  NTuple::Array<float> m_h_truth_decayVertex_t;
  NTuple::Array<float> m_h_truth_decayVertex_perp;
  NTuple::Array<float> m_h_truth_decayVertex_eta;
  NTuple::Array<float> m_h_truth_decayVertex_phi;
  NTuple::Array<float> m_h_truth_decayVertex_r;
  /// Parent/Child information
  static const unsigned int max_truth_child_num=200;
  NTuple::Array<float>  m_h_truth_child_num;
  NTuple::Matrix<long>  m_h_truth_child_Barcode;
  NTuple::Matrix<float> m_h_truth_child_pdgId;
  static const unsigned int max_truth_parent_num=200;
  NTuple::Array<float>  m_h_truth_parent_num;
  NTuple::Matrix<long>  m_h_truth_parent_Barcode;
  NTuple::Matrix<float> m_h_truth_parent_pdgId;

  // Track particles
  static const int max_track_num=500;

  NTuple::Item<long> m_h_track_num;
  NTuple::Array<float> m_h_track_pt;
  NTuple::Array<float> m_h_track_charge;
  NTuple::Array<float> m_h_track_eta;
  NTuple::Array<float> m_h_track_phi;
  NTuple::Array<long>  m_h_track_ndf;
  NTuple::Array<float> m_h_track_chi2;
  NTuple::Array<float> m_h_track_d0;
  NTuple::Array<float> m_h_track_z0;
  NTuple::Array<float> m_h_track_perigee_par0;	 
  NTuple::Array<float> m_h_track_perigee_par1;	 
  NTuple::Array<float> m_h_track_perigee_par2;	 
  NTuple::Array<float> m_h_track_perigee_par3;	 
  NTuple::Array<float> m_h_track_perigee_par4;	 
  NTuple::Array<float> m_h_track_perigee_error11;
  NTuple::Array<float> m_h_track_perigee_error22;
  NTuple::Array<float> m_h_track_perigee_error33;
  NTuple::Array<float> m_h_track_perigee_error44;
  NTuple::Array<float> m_h_track_perigee_error55;
  NTuple::Array<long>  m_h_track_origin;
  NTuple::Array<long>  m_h_track_NumBLayerHits;
  NTuple::Array<long>  m_h_track_NumPixelHits;
  NTuple::Array<long>  m_h_track_NumPixelHoles;
  NTuple::Array<long>  m_h_track_NumSCTHits;
  NTuple::Array<long>  m_h_track_NumSCTHoles;
  NTuple::Array<long>  m_h_track_NumTRTHits;
  NTuple::Array<long>  m_h_track_NumTRTHighThreshHits;
  NTuple::Array<long>  m_h_track_NumTRTHoles;
  NTuple::Array<long>  m_h_track_hitMask;

  // Particle jets ...
  // Cone
  
  static const int max_jet_num=300;
  static const int max_jettrack_num=100;

  NTuple::Item<long> m_h_cone_num;
  NTuple::Array<float> m_h_cone_pt;
  NTuple::Array<float> m_h_cone_eta;
  NTuple::Array<float> m_h_cone_phi;

  NTuple::Array<float> m_h_cone_lhsig;
  NTuple::Array<float> m_h_cone_w;
  NTuple::Array<float> m_h_cone_wIP2D;
  NTuple::Array<float> m_h_cone_wIP3D;
  NTuple::Array<float> m_h_cone_wSV1;
  NTuple::Array<float> m_h_cone_wSV2;
  NTuple::Array<float> m_h_cone_wLT1D;
  NTuple::Array<float> m_h_cone_wLT2D;
  //NTuple::Array<float> m_h_cone_wLT3D;
  NTuple::Array<float> m_h_cone_SVBU;
  //NTuple::Array<float> m_h_cone_SVTD;
  NTuple::Array<float> m_h_cone_wLHSig;
  NTuple::Array<float> m_h_cone_wSoftMu;
  NTuple::Array<float> m_h_cone_wSoftElec;
  NTuple::Array<float> m_h_cone_wJP;

  NTuple::Array<long>  m_h_cone_truth;
  NTuple::Array<long>  m_h_cone_ntracks;
  NTuple::Matrix<long> m_h_cone_trackidx;

  NTuple::Array<float> m_h_cone_sv1_pu;
  NTuple::Array<float> m_h_cone_sv1_pb;
  NTuple::Array<float> m_h_cone_sv2_pu;
  NTuple::Array<float> m_h_cone_sv2_pb;
  NTuple::Array<long>  m_h_cone_svplus_n2t; 
  NTuple::Array<float> m_h_cone_svplus_eFrac;
  NTuple::Array<float> m_h_cone_svplus_mass;
  NTuple::Array<float> m_h_cone_svplus_normDist;
  NTuple::Array<long>  m_h_cone_svplus_NGTracks;
  NTuple::Array<float> m_h_cone_svplus_chi2;
  NTuple::Array<long>  m_h_cone_svplus_ndf;
  NTuple::Array<float> m_h_cone_svplus_x;
  NTuple::Array<float> m_h_cone_svplus_y;
  NTuple::Array<float> m_h_cone_svplus_z;
  NTuple::Array<long>  m_h_cone_svplus_ntracks;
  NTuple::Matrix<long> m_h_cone_svplus_trackidx;
  NTuple::Array<float> m_h_cone_svbu_pu;
  NTuple::Array<float> m_h_cone_svbu_pb;
  NTuple::Array<float> m_h_cone_svbu_eFrac;
  NTuple::Array<float> m_h_cone_svbu_mass;
  NTuple::Array<float> m_h_cone_svbu_normDist;
  NTuple::Array<long>  m_h_cone_svbu_mult;
  NTuple::Array<float> m_h_cone_svbu_chi2;
  NTuple::Array<long>  m_h_cone_svbu_ndf;
  NTuple::Array<float> m_h_cone_svbu_x;
  NTuple::Array<float> m_h_cone_svbu_y;
  NTuple::Array<float> m_h_cone_svbu_z;         

  NTuple::Array<float> m_h_cone_ip2d_pu;
  NTuple::Array<float> m_h_cone_ip2d_pb;
  NTuple::Array<long>  m_h_cone_ip2d_numTracks;
  NTuple::Array<float> m_h_cone_ip3d_pu;
  NTuple::Array<float> m_h_cone_ip3d_pb;
  NTuple::Array<long>  m_h_cone_ip3d_numTracks;

  NTuple::Matrix<float> m_h_cone_ipplus_d0;
  NTuple::Matrix<float> m_h_cone_ipplus_d0Sig;
  NTuple::Matrix<float> m_h_cone_ipplus_z0;
  NTuple::Matrix<float> m_h_cone_ipplus_z0Sig;
  NTuple::Matrix<float> m_h_cone_ipplus_trackW2D;
  NTuple::Matrix<float> m_h_cone_ipplus_trackW3D;
  NTuple::Matrix<long>  m_h_cone_ipplus_isFromV0;
  NTuple::Matrix<float> m_h_cone_ipplus_trackJP;
  NTuple::Matrix<long>  m_h_cone_ipplus_trackGrade;
  NTuple::Matrix<long>  m_h_cone_ipplus_trackidx;

  NTuple::Array<long>   m_h_cone_lt1d_numTracks;
  NTuple::Matrix<float> m_h_cone_lt1d_z0;
  NTuple::Matrix<float> m_h_cone_lt1d_z0Sig;
  NTuple::Array<long>   m_h_cone_lt2d_numTracks;
  NTuple::Matrix<float> m_h_cone_lt2d_d0;
  NTuple::Matrix<float> m_h_cone_lt2d_d0Sig;

  NTuple::Array<long>  m_h_cone_softmu_numTracks;
  NTuple::Matrix<float> m_h_cone_softmu_d0;
  NTuple::Matrix<float> m_h_cone_softmu_ptrel;

  NTuple::Array<long>   m_h_cone_softelec_numTracks;
  NTuple::Matrix<float> m_h_cone_softelec_d0;
  NTuple::Matrix<float> m_h_cone_softelec_ptrel;

  // Cone 4
  NTuple::Item<long> m_h_cone4_num;
  NTuple::Array<float> m_h_cone4_pt;
  NTuple::Array<float> m_h_cone4_eta;
  NTuple::Array<float> m_h_cone4_phi;

  NTuple::Array<float> m_h_cone4_lhsig;
  NTuple::Array<float> m_h_cone4_w;
  NTuple::Array<float> m_h_cone4_wIP2D;
  NTuple::Array<float> m_h_cone4_wIP3D;
  NTuple::Array<float> m_h_cone4_wSV1;
  NTuple::Array<float> m_h_cone4_wSV2;
  NTuple::Array<float> m_h_cone4_wLT1D;
  NTuple::Array<float> m_h_cone4_wLT2D;
  //NTuple::Array<float> m_h_cone4_wLT3D;
  NTuple::Array<float> m_h_cone4_SVBU;
  //NTuple::Array<float> m_h_cone4_SVTD;
  NTuple::Array<float> m_h_cone4_wLHSig;
  NTuple::Array<float> m_h_cone4_wSoftMu;
  NTuple::Array<float> m_h_cone4_wSoftElec;
  NTuple::Array<float> m_h_cone4_wJP;

  NTuple::Array<long>  m_h_cone4_truth;
  NTuple::Array<long>  m_h_cone4_ntracks;
  NTuple::Matrix<long> m_h_cone4_trackidx;

  NTuple::Array<float> m_h_cone4_sv1_pu;
  NTuple::Array<float> m_h_cone4_sv1_pb;
  NTuple::Array<float> m_h_cone4_sv2_pu;
  NTuple::Array<float> m_h_cone4_sv2_pb;
  NTuple::Array<long>  m_h_cone4_svplus_n2t; 
  NTuple::Array<float> m_h_cone4_svplus_eFrac;
  NTuple::Array<float> m_h_cone4_svplus_mass;
  NTuple::Array<float> m_h_cone4_svplus_normDist;
  NTuple::Array<long>  m_h_cone4_svplus_NGTracks;
  NTuple::Array<float> m_h_cone4_svplus_chi2;
  NTuple::Array<long>  m_h_cone4_svplus_ndf;
  NTuple::Array<long>  m_h_cone4_svplus_ntracks;
  NTuple::Matrix<long> m_h_cone4_svplus_trackidx;
  NTuple::Array<float> m_h_cone4_svplus_x;
  NTuple::Array<float> m_h_cone4_svplus_y;
  NTuple::Array<float> m_h_cone4_svplus_z;
  NTuple::Array<float> m_h_cone4_svbu_pu;
  NTuple::Array<float> m_h_cone4_svbu_pb;
  NTuple::Array<float> m_h_cone4_svbu_eFrac;
  NTuple::Array<float> m_h_cone4_svbu_mass;
  NTuple::Array<float> m_h_cone4_svbu_normDist;
  NTuple::Array<long>  m_h_cone4_svbu_mult;
  NTuple::Array<float> m_h_cone4_svbu_chi2;
  NTuple::Array<long>  m_h_cone4_svbu_ndf;
  NTuple::Array<float> m_h_cone4_svbu_x;
  NTuple::Array<float> m_h_cone4_svbu_y;
  NTuple::Array<float> m_h_cone4_svbu_z;         

  NTuple::Array<float> m_h_cone4_ip2d_pu;
  NTuple::Array<float> m_h_cone4_ip2d_pb;
  NTuple::Array<long>  m_h_cone4_ip2d_numTracks;
  NTuple::Array<float> m_h_cone4_ip3d_pu;
  NTuple::Array<float> m_h_cone4_ip3d_pb;
  NTuple::Array<long>  m_h_cone4_ip3d_numTracks;

  NTuple::Matrix<float> m_h_cone4_ipplus_d0;
  NTuple::Matrix<float> m_h_cone4_ipplus_d0Sig;
  NTuple::Matrix<float> m_h_cone4_ipplus_z0;
  NTuple::Matrix<float> m_h_cone4_ipplus_z0Sig;
  NTuple::Matrix<float> m_h_cone4_ipplus_trackW2D;
  NTuple::Matrix<float> m_h_cone4_ipplus_trackW3D;
  NTuple::Matrix<long>  m_h_cone4_ipplus_isFromV0;
  NTuple::Matrix<float> m_h_cone4_ipplus_trackJP;
  NTuple::Matrix<long>  m_h_cone4_ipplus_trackGrade;
  NTuple::Matrix<long>  m_h_cone4_ipplus_trackidx;

  NTuple::Array<long>   m_h_cone4_lt1d_numTracks;
  NTuple::Matrix<float> m_h_cone4_lt1d_z0;
  NTuple::Matrix<float> m_h_cone4_lt1d_z0Sig;
  NTuple::Array<long>   m_h_cone4_lt2d_numTracks;
  NTuple::Matrix<float> m_h_cone4_lt2d_d0;
  NTuple::Matrix<float> m_h_cone4_lt2d_d0Sig;

  NTuple::Array<long>  m_h_cone4_softmu_numTracks;
  NTuple::Matrix<float> m_h_cone4_softmu_d0;
  NTuple::Matrix<float> m_h_cone4_softmu_ptrel;

  NTuple::Array<long>   m_h_cone4_softelec_numTracks;
  NTuple::Matrix<float> m_h_cone4_softelec_d0;
  NTuple::Matrix<float> m_h_cone4_softelec_ptrel;

  // Kt
  NTuple::Item<long> m_h_kt4jet_num;
  NTuple::Array<float> m_h_kt4jet_pt;
  NTuple::Array<float> m_h_kt4jet_eta;
  NTuple::Array<float> m_h_kt4jet_phi;

  NTuple::Array<float> m_h_kt4jet_lhsig;
  NTuple::Array<float> m_h_kt4jet_w;
  NTuple::Array<float> m_h_kt4jet_wIP2D;
  NTuple::Array<float> m_h_kt4jet_wIP3D;
  NTuple::Array<float> m_h_kt4jet_wSV1;
  NTuple::Array<float> m_h_kt4jet_wSV2;
  NTuple::Array<float> m_h_kt4jet_wLT1D;
  NTuple::Array<float> m_h_kt4jet_wLT2D;
  //NTuple::Array<float> m_h_kt4jet_wLT3D;
  NTuple::Array<float> m_h_kt4jet_SVBU;
  //NTuple::Array<float> m_h_kt4jet_SVTD;
  NTuple::Array<float> m_h_kt4jet_wLHSig;
  NTuple::Array<float> m_h_kt4jet_wSoftMu;
  NTuple::Array<float> m_h_kt4jet_wSoftElec;
  NTuple::Array<float> m_h_kt4jet_wJP;

  NTuple::Array<long>  m_h_kt4jet_truth;
  NTuple::Array<long>  m_h_kt4jet_ntracks;
  NTuple::Matrix<long> m_h_kt4jet_trackidx;

  NTuple::Array<float> m_h_kt4jet_sv1_pu;
  NTuple::Array<float> m_h_kt4jet_sv1_pb;
  NTuple::Array<float> m_h_kt4jet_sv2_pu;
  NTuple::Array<float> m_h_kt4jet_sv2_pb;
  NTuple::Array<long>  m_h_kt4jet_svplus_n2t; 
  NTuple::Array<float> m_h_kt4jet_svplus_eFrac;
  NTuple::Array<float> m_h_kt4jet_svplus_mass;
  NTuple::Array<float> m_h_kt4jet_svplus_normDist;
  NTuple::Array<long>  m_h_kt4jet_svplus_NGTracks;
  NTuple::Array<float> m_h_kt4jet_svplus_chi2;
  NTuple::Array<long>  m_h_kt4jet_svplus_ndf;
  NTuple::Array<float> m_h_kt4jet_svplus_x;
  NTuple::Array<float> m_h_kt4jet_svplus_y;
  NTuple::Array<float> m_h_kt4jet_svplus_z;
  NTuple::Array<long>  m_h_kt4jet_svplus_ntracks;
  NTuple::Matrix<long> m_h_kt4jet_svplus_trackidx;
  NTuple::Array<float> m_h_kt4jet_svbu_pu;
  NTuple::Array<float> m_h_kt4jet_svbu_pb;
  NTuple::Array<float> m_h_kt4jet_svbu_eFrac;
  NTuple::Array<float> m_h_kt4jet_svbu_mass;
  NTuple::Array<float> m_h_kt4jet_svbu_normDist;
  NTuple::Array<long>  m_h_kt4jet_svbu_mult;
  NTuple::Array<float> m_h_kt4jet_svbu_chi2;
  NTuple::Array<long>  m_h_kt4jet_svbu_ndf;
  NTuple::Array<float> m_h_kt4jet_svbu_x;
  NTuple::Array<float> m_h_kt4jet_svbu_y;
  NTuple::Array<float> m_h_kt4jet_svbu_z;         

  NTuple::Array<float> m_h_kt4jet_ip2d_pu;
  NTuple::Array<float> m_h_kt4jet_ip2d_pb;
  NTuple::Array<long>  m_h_kt4jet_ip2d_numTracks;
  NTuple::Array<float> m_h_kt4jet_ip3d_pu;
  NTuple::Array<float> m_h_kt4jet_ip3d_pb;
  NTuple::Array<long>  m_h_kt4jet_ip3d_numTracks;

  NTuple::Matrix<float> m_h_kt4jet_ipplus_d0;
  NTuple::Matrix<float> m_h_kt4jet_ipplus_d0Sig;
  NTuple::Matrix<float> m_h_kt4jet_ipplus_z0;
  NTuple::Matrix<float> m_h_kt4jet_ipplus_z0Sig;
  NTuple::Matrix<float> m_h_kt4jet_ipplus_trackW2D;
  NTuple::Matrix<float> m_h_kt4jet_ipplus_trackW3D;
  NTuple::Matrix<long>  m_h_kt4jet_ipplus_isFromV0;
  NTuple::Matrix<float> m_h_kt4jet_ipplus_trackJP;
  NTuple::Matrix<long>  m_h_kt4jet_ipplus_trackGrade;
  NTuple::Matrix<long>  m_h_kt4jet_ipplus_trackidx;

  NTuple::Array<long>   m_h_kt4jet_lt1d_numTracks;
  NTuple::Matrix<float> m_h_kt4jet_lt1d_z0;
  NTuple::Matrix<float> m_h_kt4jet_lt1d_z0Sig;
  NTuple::Array<long>   m_h_kt4jet_lt2d_numTracks;
  NTuple::Matrix<float> m_h_kt4jet_lt2d_d0;
  NTuple::Matrix<float> m_h_kt4jet_lt2d_d0Sig;

  NTuple::Array<long>  m_h_kt4jet_softmu_numTracks;
  NTuple::Matrix<float> m_h_kt4jet_softmu_d0;
  NTuple::Matrix<float> m_h_kt4jet_softmu_ptrel;

  NTuple::Array<long>   m_h_kt4jet_softelec_numTracks;
  NTuple::Matrix<float> m_h_kt4jet_softelec_d0;
  NTuple::Matrix<float> m_h_kt4jet_softelec_ptrel;

  // Truth particle jets ...
  // Cone
  NTuple::Item<long> m_h_cone_true_num;
  NTuple::Array<float> m_h_cone_true_pt;
  NTuple::Array<float> m_h_cone_true_eta;
  NTuple::Array<float> m_h_cone_true_phi;
  // Cone 4
  NTuple::Item<long> m_h_cone4_true_num;
  NTuple::Array<float> m_h_cone4_true_pt;
  NTuple::Array<float> m_h_cone4_true_eta;
  NTuple::Array<float> m_h_cone4_true_phi;
  // Kt
  NTuple::Item<long> m_h_kt4jet_true_num;
  NTuple::Array<float> m_h_kt4jet_true_pt;
  NTuple::Array<float> m_h_kt4jet_true_eta;
  NTuple::Array<float> m_h_kt4jet_true_phi;

  // Et miss
  NTuple::Item<float> m_h_etmiss_final_sumet;
  NTuple::Item<float> m_h_etmiss_final_ex;
  NTuple::Item<float> m_h_etmiss_final_ey;
  NTuple::Item<float> m_h_etmiss_final_phi;
  NTuple::Item<float> m_h_etmiss_final_et;
  NTuple::Item<long>  m_h_etmiss_final_source;
  NTuple::Item<long>  m_h_etmiss_final_region;
  NTuple::Item<float> m_h_etmiss_final_region_ex;
  NTuple::Item<float> m_h_etmiss_final_region_ey;
  NTuple::Item<float> m_h_etmiss_final_region_etsum;

  //Et miss Ref Final //ED start
  NTuple::Item<float> m_h_etmiss_reffinal_sumet;
  NTuple::Item<float> m_h_etmiss_reffinal_ex;
  NTuple::Item<float> m_h_etmiss_reffinal_ey;
  NTuple::Item<float> m_h_etmiss_reffinal_phi;
  NTuple::Item<float> m_h_etmiss_reffinal_et;

  //Et miss lochadtopoobj
  NTuple::Item<float> m_h_etmiss_lochadtopoobj_sumet;
  NTuple::Item<float> m_h_etmiss_lochadtopoobj_ex;
  NTuple::Item<float> m_h_etmiss_lochadtopoobj_ey;
  NTuple::Item<float> m_h_etmiss_lochadtopoobj_phi;
  NTuple::Item<float> m_h_etmiss_lochadtopoobj_et;

  //Et miss obj
  NTuple::Item<float> m_h_etmiss_objfinal_sumet;
  NTuple::Item<float> m_h_etmiss_objfinal_ex;
  NTuple::Item<float> m_h_etmiss_objfinal_ey;
  NTuple::Item<float> m_h_etmiss_objfinal_phi;
  NTuple::Item<float> m_h_etmiss_objfinal_et;
  //ED end

  // Truth Et miss
  NTuple::Item<float> m_h_etmiss_true_sumet;
  NTuple::Item<float> m_h_etmiss_true_ex;
  NTuple::Item<float> m_h_etmiss_true_ey;
  NTuple::Item<float> m_h_etmiss_true_phi;
  NTuple::Item<float> m_h_etmiss_true_et;

  // Primary Vertex
  static const int     max_vertex_num=100;
  NTuple::Item<long>   m_h_primaryvtx_num;
  NTuple::Array<float> m_h_primaryvtx_x;
  NTuple::Array<float> m_h_primaryvtx_y;
  NTuple::Array<float> m_h_primaryvtx_z;
  NTuple::Array<float> m_h_primaryvtx_dX;
  NTuple::Array<float> m_h_primaryvtx_dY;
  NTuple::Array<float> m_h_primaryvtx_dZ;
  NTuple::Array<float> m_h_primaryvtx_chi2;
  NTuple::Array<long>  m_h_primaryvtx_ndf;
 
};
   
#endif // ANALYSIS_SKELETON_H

